![MikroE](http://www.mikroe.com/img/designs/beta/logo_small.png)

---

# SecureSWI Click

---

- **CIC Prefix**  : SECURESWI
- **Author**      : Dusan Poluga
- **Verison**     : 1.0.0
- **Date**        : apr 2019.

---

- **Libstock** : https://libstock.mikroe.com/projects/view/2880/secure-5-click 
- **HEXIWEAR** : ${HEXI_LINK}
- **GitHub**   : ${GITHUB_LINK}

---

### Software Support

We provide a library for the SecureSWI Click on our [LibStock](https://libstock.mikroe.com/projects/view/2880/secure-5-click)
page, as well as a demo application (example), developed using MikroElektronika 
[compilers](http://shop.mikroe.com/compilers). The demo can run on all the main 
MikroElektronika [development boards](http://shop.mikroe.com/development-boards).

**Library Description**

The library demonstrates the operation of the software single wire interface implementation.

Key functions :

- ``` int8_t secureswi_init(T_SECURESWI_DIRSET inSet, T_SECURESWI_DIRSET outSet) ``` - Initialize the SWI interface and pass the pin direction setting functions.
- ``` void secureswi_sendBytes(uint8_t len,uint8_t *stBuf) ``` - Encode data buffer and send the data to the SWI bus.
- ``` void secureswi_receiveBytes(uint8_t len,uint8_t *stBuf) ``` - Receive and decode data from the SWI bus.

**Examples Description**

The application is composed of three sections :

- System Initialization - Initialize the GPIO sturcture and configure the serial
                          port for logging data.
- Application Initialization - Initialize the driver and configure swi for
                               communication.
- Application Task - Data is read from the secure chip. If the readout is
                     successful the data is then printed on the serial port in
                     the hex format.


```.c
void applicationTask()
{
     uint8_t bufferOut[128];

     cfg_atsha204a_swi_default.iface_type  = ATCA_SWI_IFACE;
     cfg_atsha204a_swi_default.devtype     = ATSHA204A;
     cfg_atsha204a_swi_default.atcaswi.bus = 1;
     cfg_atsha204a_swi_default.wake_delay  = 2560;
     cfg_atsha204a_swi_default.rx_retries  = 10;

     atcab_init(&cfg_atsha204a_swi_default);

     mikrobus_logWrite("Starting test",_LOG_LINE);

     memset(bufferOut,0,127);

     if (atcab_read_serial_number(bufferOut) == ATCA_SUCCESS)
     {
         mikrobus_logWrite("\r\n Serial number: ",_LOG_LINE);
         secureswi_printHex(bufferOut,9);
     }
     else
     {
         mikrobus_logWrite("\r\n Reading serial number failed...",_LOG_LINE);
         secureswi_printHex(bufferOut,sizeof(bufferOut));
     }

     Delay_ms (1500);
     memset (bufferOut, 0x00, 128);
     if (atcab_read_config_zone(bufferOut) == ATCA_SUCCESS)
     {
        mikrobus_logWrite("\r\n\r\n First 32 bytes of device configuration: ",_LOG_LINE);
        secureswi_printHex(bufferOut,32);
     }
     else
     {
        mikrobus_logWrite("\r\n\r\n Reading config zone failed...",_LOG_LINE);
        secureswi_printHex(bufferOut,sizeof(bufferOut));
     }

     while(1)
     {

     }
}
```

The full application code, and ready to use projects can be found on our 
[LibStock](https://libstock.mikroe.com/projects/view/2880/secure-5-click) page.

Other mikroE Libraries used in the example:

- Conversions
- C_String
- UART

**Additional notes and informations**

Depending on the development board you are using, you may need 
[USB UART click](http://shop.mikroe.com/usb-uart-click), 
[USB UART 2 Click](http://shop.mikroe.com/usb-uart-2-click) or 
[RS232 Click](http://shop.mikroe.com/rs232-click) to connect to your PC, for 
development systems with no UART to USB interface available on the board. The 
terminal available in all Mikroelektronika 
[compilers](http://shop.mikroe.com/compilers), or any other terminal application 
of your choice, can be used to read the message.

---
### Architectures Supported

#### mikroC

| STM | KIN | CEC | MSP | TIVA | PIC | PIC32 | DSPIC | AVR | FT90x |
|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|
| x | x | x | x | x | - | x | x | - | - |

#### mikroBasic

| STM | KIN | CEC | MSP | TIVA | PIC | PIC32 | DSPIC | AVR | FT90x |
|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|
| - | - | - | - | - | - | - | - | - | - |

#### mikroPascal

| STM | KIN | CEC | MSP | TIVA | PIC | PIC32 | DSPIC | AVR | FT90x |
|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|:-:|
| - | - | - | - | - | - | - | - | - | - |

---
---