/*
Example for SecureSWI Click

    Date          : apr 2019.
    Author        : Dusan Poluga

Test configuration PIC :
    
    MCU             : P18F87K22
    Dev. Board      : EasyPIC PRO v7
    PIC Compiler ver : v7.2.0.0

---

Description :

The application is composed of three sections :

- System Initialization - Initialize the GPIO sturcture and configure the serial 
                          port for logging data.
- Application Initialization - Initialize the driver and configure swi for 
                               communication.
- Application Task - Data is read from the secure chip. If the readout is 
                     successful the data is then printed on the serial port in 
                     the hex format.

*/

#include "Click_SecureSWI_types.h"
#include "Click_SecureSWI_config.h"
#include "__secureswi_driver.h"
#include "cryptoauthlib.h"

uint8_t otherData [ 13 ] = {0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
uint8_t checkMac [ 32 ] = {
 0xCD, 0x89, 0xA9, 0xA0, 0xD2, 0x63, 0x44, 0x9F, 0x58, 0x48, 0x81, 0xFC, 0xEF,
 0xF6, 0xD1, 0xFD, 0x82, 0xF7, 0xEF, 0xF3, 0x47, 0x4C, 0xB7, 0x1B, 0x1E, 0x9C,
 0x76, 0x13, 0x02, 0x18, 0x6E, 0x32 };

void atca_delay_us(uint32_t stat)
{
   uint32_t i;
   i=stat;
   while(i--)
   {
    Delay_1us();
   }
}

void atca_delay_ms(uint32_t stat)
{
   uint32_t i;
   i=stat;
   while(i--)
   {
    Delay_1ms();
   }
}

void setInput( void )
{
   mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_TX_PIN, _GPIO_INPUT );
}

void setOutput( void )
{
   mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_TX_PIN, _GPIO_OUTPUT );
}

void systemInit()
{
   mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_TX_PIN, _GPIO_OUTPUT );
   mikrobus_logInit(  _LOG_USBUART,  57600  );
   Delay_ms( 100 );
}

void applicationInit()
{
    secureswi_gpioDriverInit( (T_SECURESWI_P)&_MIKROBUS1_GPIO );
    secureswi_init(setInput,setOutput);
    
    mikrobus_logWrite( "Init", _LOG_LINE );
    Delay_ms(2000);
}

void secureswi_printHex(uint8_t *dat, uint16_t len)
{
    uint8_t convData[2];
    while(len--)
    {
       ByteToHex(*dat++,&convData[0]);
       mikrobus_logWrite("0x",_LOG_TEXT);
       mikrobus_logWrite(convData,_LOG_TEXT);
       mikrobus_logWrite(" ",_LOG_TEXT);
    }
    mikrobus_logWrite("\r\n",_LOG_TEXT);
}

void applicationTask()
{
     uint8_t bufferOut[128];
     
     cfg_atsha204a_swi_default.iface_type  = ATCA_SWI_IFACE;
     cfg_atsha204a_swi_default.devtype     = ATSHA204A;
     cfg_atsha204a_swi_default.atcaswi.bus = 1;
     cfg_atsha204a_swi_default.wake_delay  = 2560;
     cfg_atsha204a_swi_default.rx_retries  = 10;

     atcab_init(&cfg_atsha204a_swi_default);
     
     mikrobus_logWrite("Starting test",_LOG_LINE);
     
     memset(bufferOut,0,127);
     
     if (atcab_read_serial_number(bufferOut) == ATCA_SUCCESS)
     {
         mikrobus_logWrite("\r\n Serial number: ",_LOG_LINE);
         secureswi_printHex(bufferOut,9);
     }
     else
     {
         mikrobus_logWrite("\r\n Reading serial number failed...",_LOG_LINE);
         secureswi_printHex(bufferOut,sizeof(bufferOut));
     }
     
     Delay_ms (1500);
     memset (bufferOut, 0x00, 128);
     if (atcab_read_config_zone(bufferOut) == ATCA_SUCCESS)
     {
        mikrobus_logWrite("\r\n\r\n First 32 bytes of device configuration: ",_LOG_LINE);
        secureswi_printHex(bufferOut,32);
     }
     else
     {
        mikrobus_logWrite("\r\n\r\n Reading config zone failed...",_LOG_LINE);
        secureswi_printHex(bufferOut,sizeof(bufferOut));
     }
      
     while(1)
     {

     }
}

void main()
{
    systemInit();
    applicationInit();

    while (1)
    {
            applicationTask();
    }
}