/*
    __secureswi_driver.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

#include "__secureswi_driver.h"
#include "__secureswi_hal.c"

/* ------------------------------------------------------------------- MACROS */
static const uint32_t SECURESWI_WORKBAUD = 230400;
static const uint32_t SECURESWI_INSTR_PER_CLOCK = 1;
static       uint32_t SECURESWI_BIT_DELAY;
static       uint32_t SECURESWI_HALF_BIT_DELAY;


const int8_t POINTER_ZERO = -1;
const int8_t INIT_OK      =  0;

const uint8_t SWIPIN_INPUT  = 1;
const uint8_t SWIPIN_OUTPUT = 0;

static uint8_t tempStorage[1024];


/* ---------------------------------------------------------------- VARIABLES */
T_SECURESWI_DIRSET SECURESWI_DIRIN;
T_SECURESWI_DIRSET SECURESWI_DIROUT;

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */



/* --------------------------------------------- PRIVATE FUNCTION DEFINITIONS */

static void secureswi_calcBaud()
{
     uint32_t tmp,tmp1;

     if(tmp);
     tmp  = Get_Fosc_kHz() * 1000;
     tmp1 = SECURESWI_WORKBAUD * SECURESWI_INSTR_PER_CLOCK * 10;

     SECURESWI_BIT_DELAY = ( tmp / tmp1 ) - 8;    //8

     tmp %= tmp1;

     if (tmp > tmp1>>1)
     {
      SECURESWI_BIT_DELAY++;
     }

     SECURESWI_HALF_BIT_DELAY = SECURESWI_BIT_DELAY>>1;
     
     memset(tempStorage,0,512);
}

/* --------------------------------------------------------- PUBLIC FUNCTIONS */

void secureswi_gpioDriverInit(T_SECURESWI_P gpioObj)
{
    hal_gpioMap( (T_HAL_P)gpioObj );
}

static void secureswi_pinWr(uint8_t state)
{
      hal_gpio_txSet(state);
}

static uint8_t secureswi_pinRd()
{
    return hal_gpio_txGet();
}

/* ----------------------------------------------------------- IMPLEMENTATION */

void secureswi_pinState(uint8_t state)
{
      hal_gpio_txSet(state);
}

int8_t secureswi_init(T_SECURESWI_DIRSET inSet, T_SECURESWI_DIRSET outSet)
{
     if(SECURESWI_WORKBAUD);
     if(SECURESWI_INSTR_PER_CLOCK);
     if(SECURESWI_BIT_DELAY);
     if(SECURESWI_HALF_BIT_DELAY);

     if(inSet == 0)  return POINTER_ZERO;
     if(outSet == 0) return POINTER_ZERO;

     SECURESWI_DIRIN  = inSet;
     SECURESWI_DIROUT = outSet;
     
     hal_gpio_txSet(1);
     secureswi_calcBaud();
}

void secureswi_sendBit(uint8_t _tx)
{
      uint8_t i=7,j=0;
      uint8_t _txSend;
      _txSend = _tx;
      SECURESWI_DIROUT();

      hal_gpio_txSet(0);

      while( i-- )
      {
          #ifdef __MIKROC_PRO_FOR_DSPIC__
          Delay_Cyc_Long( SECURESWI_BIT_DELAY + 2 );
          #else
          Delay_Cyc( SECURESWI_BIT_DELAY + 2 );
          #endif

          if( _txSend & ( 1<<j ) )
          {
             hal_gpio_txSet(1);
          }
          else
          {
             hal_gpio_txSet(0);
          }
          j++;

      }
      #ifdef __MIKROC_PRO_FOR_DSPIC__
      Delay_Cyc_Long( SECURESWI_BIT_DELAY );
      #else
      Delay_Cyc( SECURESWI_BIT_DELAY );
      #endif
      hal_gpio_txSet(1); //stop
      #ifdef __MIKROC_PRO_FOR_DSPIC__
      Delay_Cyc_Long( SECURESWI_BIT_DELAY );
      #else
      Delay_Cyc( SECURESWI_BIT_DELAY );
      #endif
}

uint8_t secureswi_receiveBit()
{
   uint8_t tmp=0;
   uint8_t mask=1;
   uint8_t i=7;
   uint32_t braketimer=4200000;
   SECURESWI_DIRIN();
   if(tmp);
   while( hal_gpio_txGet() == 1 )
   {
          if(braketimer == 0)
          {
            return 0;
          }
          else
          {
             braketimer--;
          }
   }

      #ifdef __MIKROC_PRO_FOR_DSPIC__
      Delay_Cyc_Long( SECURESWI_BIT_DELAY );
      #else
      Delay_Cyc( SECURESWI_BIT_DELAY );
      #endif
   while(i--)
   {
      #ifdef __MIKROC_PRO_FOR_DSPIC__
      Delay_Cyc_Long( SECURESWI_BIT_DELAY );
      #else
      Delay_Cyc( SECURESWI_BIT_DELAY );
      #endif
     if( hal_gpio_txGet() == 1 )
     {
       tmp |= mask;
     }
     mask <<= 1;
   }
      #ifdef __MIKROC_PRO_FOR_DSPIC__
      Delay_Cyc_Long( SECURESWI_BIT_DELAY );
      #else
      Delay_Cyc( SECURESWI_BIT_DELAY );
      #endif
      #ifdef __MIKROC_PRO_FOR_DSPIC__
      Delay_Cyc_Long( SECURESWI_BIT_DELAY );
      #else
      Delay_Cyc( SECURESWI_BIT_DELAY );
      #endif
   return tmp;
}

void secureswi_wakeDevice()
{
    hal_gpio_txSet(0);
    
    #ifdef __MIKROC_PRO_FOR_DSPIC__
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    Delay_Cyc_Long(SECURESWI_BIT_DELAY);
    #else
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    Delay_Cyc(SECURESWI_BIT_DELAY);
    #endif
    
    hal_gpio_txSet(1);
    #ifdef __MIKROC_PRO_FOR_DSPIC__
    Delay_Cyc_Long( SECURESWI_BIT_DELAY );
    #else
    Delay_Cyc( SECURESWI_BIT_DELAY );
    #endif
}

void secureswi_decode(uint8_t *in, uint8_t *out)
{
   uint8_t cn=0;

   while(cn != 8)
   {
      if(*in == 0x7F)
      {
         *out |= 1<<cn;
      }
      else
      {

      }
      cn++;
      in++;
   }
}

void secureswi_receiveBytes(uint8_t len,uint8_t *stBuf)
{
    uint16_t rCounter = 0;
    uint16_t tLen = 0;
    uint16_t bLen = 0;
    uint16_t offset = 0;

    tLen = len*8;
    while(tLen--)
    {
       tempStorage[rCounter] = 0;
       tempStorage[rCounter]=secureswi_receiveBit();
       if(tempStorage[rCounter] != 0)
       {
        rCounter++;
       }
       else
       {
        break;
       }
    }

    tLen = len;
    while(tLen--)
    {
       stBuf[bLen] = 0;
       secureswi_decode( &tempStorage[offset] ,&stBuf[bLen] );
       offset += 8;
       bLen++;
    }
}

void secureswi_encodeByte(uint8_t *to, uint8_t from)
{
       uint8_t i = 8;
       uint8_t j = 0;

       while(i--)
       {
          if( from & (1<<j) )
          {
              to[j] = 0x7F;
          }
          else
          {
              to[j] = 0x7D;
          }
          j++;
       }
}

void secureswi_sendBytes(uint8_t len, uint8_t *stBuf)
{
    uint8_t tCounter = 0;
    uint8_t tLen = 0;
    uint8_t bLen = 0;
    uint8_t encodeBuff[255];
    uint8_t offset=0;

    memset(encodeBuff,0,255);

    tLen = len;
    while(tLen--)
    {
      secureswi_encodeByte(&encodeBuff[offset], stBuf[bLen]);
      offset += 8;
      bLen++;
    }
    tLen = 8 * len;

    while(tLen--)
    {
        secureswi_sendBit(encodeBuff[tCounter]);
        tCounter++;
    }
}

/* -------------------------------------------------------------------------- */
/*
  __secureswi_driver.c

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by the MikroElektonika.

4. Neither the name of the MikroElektonika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */