/*
    __secureswi_driver.h

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
@file   __secureswi_driver.h
@brief    SecureSWI Driver
@mainpage SecureSWI Click
@{

@image html libstock_fb_view.jpg

@}

@defgroup   SECURESWI
@brief      SecureSWI Click Driver
@{

| Global Library Prefix | **SECURESWI** |
|:---------------------:|:-----------------:|
| Version               | **1.0.0**    |
| Date                  | **apr 2019.**      |
| Developer             | **Dusan Poluga**     |

*/
/* -------------------------------------------------------------------------- */

#include "stdint.h"

#ifndef _SECURESWI_H_
#define _SECURESWI_H_

/** 
 * @macro T_SECURESWI_P
 * @brief Driver Abstract type 
 */
#define T_SECURESWI_P    const uint8_t*

typedef void (*T_SECURESWI_DIRSET)( void );

/** @defgroup SECURESWI_COMPILE Compilation Config */              /** @{ */


                                                                       /** @} */
/** @defgroup SECURESWI_VAR Variables */                           /** @{ */
extern T_SECURESWI_DIRSET SECURESWI_DIRIN;
extern T_SECURESWI_DIRSET SECURESWI_DIROUT;
                                                                       /** @} */
/** @defgroup SECURESWI_TYPES Types */                             /** @{ */



                                                                       /** @} */
#ifdef __cplusplus
extern "C"{
#endif

/** @defgroup SECURESWI_INIT Driver Initialization */              /** @{ */
void secureswi_gpioDriverInit(T_SECURESWI_P gpioObj);

/**
 * Initialize the bitbang swi interface and pass the direction setting functions
 */
int8_t secureswi_init(T_SECURESWI_DIRSET inSet, T_SECURESWI_DIRSET outSet);

/**
 * Send a raw non necoded bit to the swi bus.
 */
void secureswi_sendBit(uint8_t _tx);

/**
 * Receive a encoded bit from the swi bus.
 */
uint8_t secureswi_receiveBit();

/**
 * Send a wakeup signal to the swi bus.
 */
void secureswi_wakeDevice();

/**
 * Decode bits received from the swi bus.
 */
void secureswi_decode(uint8_t *in, uint8_t *out);

/**
 * Receive multiple bytes of data from the swi bus.]
 */
void secureswi_receiveBytes(uint8_t len,uint8_t *stBuf);

/**
 * Encode byte function will encode every bit of data with
 * 0x7F for 1 and 0x7D for 0. 
 */
void secureswi_encodeByte(uint8_t *to, uint8_t from);

/**
 * Function for sending n number of bytes to the swi bus.
 */
void secureswi_sendBytes(uint8_t len,uint8_t *stBuf);

/**
 * Function for setting the pin state.
 */
void secureswi_pinState(uint8_t state);

                                                                       /** @} */
/** @defgroup SECURESWI_FUNC Driver Functions */                   /** @{ */







                                                                       /** @} */
#ifdef __cplusplus
} // extern "C"
#endif
#endif

/**
    @example Click_SecureSWI_STM.c
    @example Click_SecureSWI_TIVA.c
    @example Click_SecureSWI_CEC.c
    @example Click_SecureSWI_KINETIS.c
    @example Click_SecureSWI_MSP.c
    @example Click_SecureSWI_PIC.c
    @example Click_SecureSWI_PIC32.c
    @example Click_SecureSWI_DSPIC.c
    @example Click_SecureSWI_AVR.c
    @example Click_SecureSWI_FT90x.c
    @example Click_SecureSWI_STM.mbas
    @example Click_SecureSWI_TIVA.mbas
    @example Click_SecureSWI_CEC.mbas
    @example Click_SecureSWI_KINETIS.mbas
    @example Click_SecureSWI_MSP.mbas
    @example Click_SecureSWI_PIC.mbas
    @example Click_SecureSWI_PIC32.mbas
    @example Click_SecureSWI_DSPIC.mbas
    @example Click_SecureSWI_AVR.mbas
    @example Click_SecureSWI_FT90x.mbas
    @example Click_SecureSWI_STM.mpas
    @example Click_SecureSWI_TIVA.mpas
    @example Click_SecureSWI_CEC.mpas
    @example Click_SecureSWI_KINETIS.mpas
    @example Click_SecureSWI_MSP.mpas
    @example Click_SecureSWI_PIC.mpas
    @example Click_SecureSWI_PIC32.mpas
    @example Click_SecureSWI_DSPIC.mpas
    @example Click_SecureSWI_AVR.mpas
    @example Click_SecureSWI_FT90x.mpas
*/                                                                     /** @} */
/* -------------------------------------------------------------------------- */
/*
  __secureswi_driver.h

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by the MikroElektonika.

4. Neither the name of the MikroElektonika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */
